# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fedn/common/net/grpc/fedn.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x66\x65\x64n/common/net/grpc/fedn.proto\x12\x04grpc\":\n\x08Response\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x10\n\x08response\x18\x02 \x01(\t\"\x8c\x02\n\x06Status\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x0e\n\x06status\x18\x02 \x01(\t\x12(\n\tlog_level\x18\x03 \x01(\x0e\x32\x15.grpc.Status.LogLevel\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t\x12\x16\n\x0e\x63orrelation_id\x18\x05 \x01(\t\x12\x11\n\ttimestamp\x18\x06 \x01(\t\x12\x1e\n\x04type\x18\x07 \x01(\x0e\x32\x10.grpc.StatusType\x12\r\n\x05\x65xtra\x18\x08 \x01(\t\"B\n\x08LogLevel\x12\x08\n\x04INFO\x10\x00\x12\t\n\x05\x44\x45\x42UG\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x12\t\n\x05\x41UDIT\x10\x04\"\xab\x01\n\x12ModelUpdateRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x10\n\x08model_id\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t\x12\x16\n\x0e\x63orrelation_id\x18\x05 \x01(\t\x12\x11\n\ttimestamp\x18\x06 \x01(\t\x12\x0c\n\x04meta\x18\x07 \x01(\t\"\xaf\x01\n\x0bModelUpdate\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x10\n\x08model_id\x18\x03 \x01(\t\x12\x17\n\x0fmodel_update_id\x18\x04 \x01(\t\x12\x16\n\x0e\x63orrelation_id\x18\x05 \x01(\t\x12\x11\n\ttimestamp\x18\x06 \x01(\t\x12\x0c\n\x04meta\x18\x07 \x01(\t\"\xc5\x01\n\x16ModelValidationRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x10\n\x08model_id\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t\x12\x16\n\x0e\x63orrelation_id\x18\x05 \x01(\t\x12\x11\n\ttimestamp\x18\x06 \x01(\t\x12\x0c\n\x04meta\x18\x07 \x01(\t\x12\x14\n\x0cis_inference\x18\x08 \x01(\x08\"\xa8\x01\n\x0fModelValidation\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x10\n\x08model_id\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t\x12\x16\n\x0e\x63orrelation_id\x18\x05 \x01(\t\x12\x11\n\ttimestamp\x18\x06 \x01(\t\x12\x0c\n\x04meta\x18\x07 \x01(\t\"\x89\x01\n\x0cModelRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x0c\n\x04\x64\x61ta\x18\x03 \x01(\x0c\x12\n\n\x02id\x18\x04 \x01(\t\x12!\n\x06status\x18\x05 \x01(\x0e\x32\x11.grpc.ModelStatus\"]\n\rModelResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\n\n\x02id\x18\x02 \x01(\t\x12!\n\x06status\x18\x03 \x01(\x0e\x32\x11.grpc.ModelStatus\x12\x0f\n\x07message\x18\x04 \x01(\t\"U\n\x15GetGlobalModelRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\"h\n\x16GetGlobalModelResponse\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x10\n\x08model_id\x18\x03 \x01(\t\")\n\tHeartbeat\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\"W\n\x16\x43lientAvailableMessage\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\x12\x11\n\ttimestamp\x18\x03 \x01(\t\"R\n\x12ListClientsRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x07\x63hannel\x18\x02 \x01(\x0e\x32\r.grpc.Channel\"*\n\nClientList\x12\x1c\n\x06\x63lient\x18\x01 \x03(\x0b\x32\x0c.grpc.Client\"0\n\x06\x43lient\x12\x18\n\x04role\x18\x01 \x01(\x0e\x32\n.grpc.Role\x12\x0c\n\x04name\x18\x02 \x01(\t\"m\n\x0fReassignRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x0e\n\x06server\x18\x03 \x01(\t\x12\x0c\n\x04port\x18\x04 \x01(\r\"c\n\x10ReconnectRequest\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\x1e\n\x08receiver\x18\x02 \x01(\x0b\x32\x0c.grpc.Client\x12\x11\n\treconnect\x18\x03 \x01(\r\"\'\n\tParameter\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"T\n\x0e\x43ontrolRequest\x12\x1e\n\x07\x63ommand\x18\x01 \x01(\x0e\x32\r.grpc.Command\x12\"\n\tparameter\x18\x02 \x03(\x0b\x32\x0f.grpc.Parameter\"F\n\x0f\x43ontrolResponse\x12\x0f\n\x07message\x18\x01 \x01(\t\x12\"\n\tparameter\x18\x02 \x03(\x0b\x32\x0f.grpc.Parameter\"R\n\x0eReportResponse\x12\x1c\n\x06sender\x18\x01 \x01(\x0b\x32\x0c.grpc.Client\x12\"\n\tparameter\x18\x02 \x03(\x0b\x32\x0f.grpc.Parameter\"\x13\n\x11\x43onnectionRequest\"<\n\x12\x43onnectionResponse\x12&\n\x06status\x18\x01 \x01(\x0e\x32\x16.grpc.ConnectionStatus*\x84\x01\n\nStatusType\x12\x07\n\x03LOG\x10\x00\x12\x18\n\x14MODEL_UPDATE_REQUEST\x10\x01\x12\x10\n\x0cMODEL_UPDATE\x10\x02\x12\x1c\n\x18MODEL_VALIDATION_REQUEST\x10\x03\x12\x14\n\x10MODEL_VALIDATION\x10\x04\x12\r\n\tINFERENCE\x10\x05*\x86\x01\n\x07\x43hannel\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x19\n\x15MODEL_UPDATE_REQUESTS\x10\x01\x12\x11\n\rMODEL_UPDATES\x10\x02\x12\x1d\n\x19MODEL_VALIDATION_REQUESTS\x10\x03\x12\x15\n\x11MODEL_VALIDATIONS\x10\x04\x12\n\n\x06STATUS\x10\x05*F\n\x0bModelStatus\x12\x06\n\x02OK\x10\x00\x12\x0f\n\x0bIN_PROGRESS\x10\x01\x12\x12\n\x0eIN_PROGRESS_OK\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03*8\n\x04Role\x12\n\n\x06WORKER\x10\x00\x12\x0c\n\x08\x43OMBINER\x10\x01\x12\x0b\n\x07REDUCER\x10\x02\x12\t\n\x05OTHER\x10\x03*J\n\x07\x43ommand\x12\x08\n\x04IDLE\x10\x00\x12\t\n\x05START\x10\x01\x12\t\n\x05PAUSE\x10\x02\x12\x08\n\x04STOP\x10\x03\x12\t\n\x05RESET\x10\x04\x12\n\n\x06REPORT\x10\x05*I\n\x10\x43onnectionStatus\x12\x11\n\rNOT_ACCEPTING\x10\x00\x12\r\n\tACCEPTING\x10\x01\x12\x13\n\x0fTRY_AGAIN_LATER\x10\x02\x32z\n\x0cModelService\x12\x33\n\x06Upload\x12\x12.grpc.ModelRequest\x1a\x13.grpc.ModelResponse(\x01\x12\x35\n\x08\x44ownload\x12\x12.grpc.ModelRequest\x1a\x13.grpc.ModelResponse0\x01\x32\xa9\x02\n\x07\x43ontrol\x12\x34\n\x05Start\x12\x14.grpc.ControlRequest\x1a\x15.grpc.ControlResponse\x12\x33\n\x04Stop\x12\x14.grpc.ControlRequest\x1a\x15.grpc.ControlResponse\x12\x37\n\tConfigure\x12\x14.grpc.ControlRequest\x1a\x14.grpc.ReportResponse\x12\x44\n\x15\x46lushAggregationQueue\x12\x14.grpc.ControlRequest\x1a\x15.grpc.ControlResponse\x12\x34\n\x06Report\x12\x14.grpc.ControlRequest\x1a\x14.grpc.ReportResponse2V\n\x07Reducer\x12K\n\x0eGetGlobalModel\x12\x1b.grpc.GetGlobalModelRequest\x1a\x1c.grpc.GetGlobalModelResponse2\xab\x03\n\tConnector\x12\x44\n\x14\x41llianceStatusStream\x12\x1c.grpc.ClientAvailableMessage\x1a\x0c.grpc.Status0\x01\x12*\n\nSendStatus\x12\x0c.grpc.Status\x1a\x0e.grpc.Response\x12?\n\x11ListActiveClients\x12\x18.grpc.ListClientsRequest\x1a\x10.grpc.ClientList\x12\x45\n\x10\x41\x63\x63\x65ptingClients\x12\x17.grpc.ConnectionRequest\x1a\x18.grpc.ConnectionResponse\x12\x30\n\rSendHeartbeat\x12\x0f.grpc.Heartbeat\x1a\x0e.grpc.Response\x12\x37\n\x0eReassignClient\x12\x15.grpc.ReassignRequest\x1a\x0e.grpc.Response\x12\x39\n\x0fReconnectClient\x12\x16.grpc.ReconnectRequest\x1a\x0e.grpc.Response2\xda\x04\n\x08\x43ombiner\x12T\n\x18ModelUpdateRequestStream\x12\x1c.grpc.ClientAvailableMessage\x1a\x18.grpc.ModelUpdateRequest0\x01\x12\x46\n\x11ModelUpdateStream\x12\x1c.grpc.ClientAvailableMessage\x1a\x11.grpc.ModelUpdate0\x01\x12\\\n\x1cModelValidationRequestStream\x12\x1c.grpc.ClientAvailableMessage\x1a\x1c.grpc.ModelValidationRequest0\x01\x12N\n\x15ModelValidationStream\x12\x1c.grpc.ClientAvailableMessage\x1a\x15.grpc.ModelValidation0\x01\x12\x42\n\x16SendModelUpdateRequest\x12\x18.grpc.ModelUpdateRequest\x1a\x0e.grpc.Response\x12\x34\n\x0fSendModelUpdate\x12\x11.grpc.ModelUpdate\x1a\x0e.grpc.Response\x12J\n\x1aSendModelValidationRequest\x12\x1c.grpc.ModelValidationRequest\x1a\x0e.grpc.Response\x12<\n\x13SendModelValidation\x12\x15.grpc.ModelValidation\x1a\x0e.grpc.Responseb\x06proto3')

_STATUSTYPE = DESCRIPTOR.enum_types_by_name['StatusType']
StatusType = enum_type_wrapper.EnumTypeWrapper(_STATUSTYPE)
_CHANNEL = DESCRIPTOR.enum_types_by_name['Channel']
Channel = enum_type_wrapper.EnumTypeWrapper(_CHANNEL)
_MODELSTATUS = DESCRIPTOR.enum_types_by_name['ModelStatus']
ModelStatus = enum_type_wrapper.EnumTypeWrapper(_MODELSTATUS)
_ROLE = DESCRIPTOR.enum_types_by_name['Role']
Role = enum_type_wrapper.EnumTypeWrapper(_ROLE)
_COMMAND = DESCRIPTOR.enum_types_by_name['Command']
Command = enum_type_wrapper.EnumTypeWrapper(_COMMAND)
_CONNECTIONSTATUS = DESCRIPTOR.enum_types_by_name['ConnectionStatus']
ConnectionStatus = enum_type_wrapper.EnumTypeWrapper(_CONNECTIONSTATUS)
LOG = 0
MODEL_UPDATE_REQUEST = 1
MODEL_UPDATE = 2
MODEL_VALIDATION_REQUEST = 3
MODEL_VALIDATION = 4
INFERENCE = 5
DEFAULT = 0
MODEL_UPDATE_REQUESTS = 1
MODEL_UPDATES = 2
MODEL_VALIDATION_REQUESTS = 3
MODEL_VALIDATIONS = 4
STATUS = 5
OK = 0
IN_PROGRESS = 1
IN_PROGRESS_OK = 2
FAILED = 3
WORKER = 0
COMBINER = 1
REDUCER = 2
OTHER = 3
IDLE = 0
START = 1
PAUSE = 2
STOP = 3
RESET = 4
REPORT = 5
NOT_ACCEPTING = 0
ACCEPTING = 1
TRY_AGAIN_LATER = 2


_RESPONSE = DESCRIPTOR.message_types_by_name['Response']
_STATUS = DESCRIPTOR.message_types_by_name['Status']
_MODELUPDATEREQUEST = DESCRIPTOR.message_types_by_name['ModelUpdateRequest']
_MODELUPDATE = DESCRIPTOR.message_types_by_name['ModelUpdate']
_MODELVALIDATIONREQUEST = DESCRIPTOR.message_types_by_name['ModelValidationRequest']
_MODELVALIDATION = DESCRIPTOR.message_types_by_name['ModelValidation']
_MODELREQUEST = DESCRIPTOR.message_types_by_name['ModelRequest']
_MODELRESPONSE = DESCRIPTOR.message_types_by_name['ModelResponse']
_GETGLOBALMODELREQUEST = DESCRIPTOR.message_types_by_name['GetGlobalModelRequest']
_GETGLOBALMODELRESPONSE = DESCRIPTOR.message_types_by_name['GetGlobalModelResponse']
_HEARTBEAT = DESCRIPTOR.message_types_by_name['Heartbeat']
_CLIENTAVAILABLEMESSAGE = DESCRIPTOR.message_types_by_name['ClientAvailableMessage']
_LISTCLIENTSREQUEST = DESCRIPTOR.message_types_by_name['ListClientsRequest']
_CLIENTLIST = DESCRIPTOR.message_types_by_name['ClientList']
_CLIENT = DESCRIPTOR.message_types_by_name['Client']
_REASSIGNREQUEST = DESCRIPTOR.message_types_by_name['ReassignRequest']
_RECONNECTREQUEST = DESCRIPTOR.message_types_by_name['ReconnectRequest']
_PARAMETER = DESCRIPTOR.message_types_by_name['Parameter']
_CONTROLREQUEST = DESCRIPTOR.message_types_by_name['ControlRequest']
_CONTROLRESPONSE = DESCRIPTOR.message_types_by_name['ControlResponse']
_REPORTRESPONSE = DESCRIPTOR.message_types_by_name['ReportResponse']
_CONNECTIONREQUEST = DESCRIPTOR.message_types_by_name['ConnectionRequest']
_CONNECTIONRESPONSE = DESCRIPTOR.message_types_by_name['ConnectionResponse']
_STATUS_LOGLEVEL = _STATUS.enum_types_by_name['LogLevel']
Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), {
  'DESCRIPTOR' : _RESPONSE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.Response)
  })
_sym_db.RegisterMessage(Response)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.Status)
  })
_sym_db.RegisterMessage(Status)

ModelUpdateRequest = _reflection.GeneratedProtocolMessageType('ModelUpdateRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELUPDATEREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ModelUpdateRequest)
  })
_sym_db.RegisterMessage(ModelUpdateRequest)

ModelUpdate = _reflection.GeneratedProtocolMessageType('ModelUpdate', (_message.Message,), {
  'DESCRIPTOR' : _MODELUPDATE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ModelUpdate)
  })
_sym_db.RegisterMessage(ModelUpdate)

ModelValidationRequest = _reflection.GeneratedProtocolMessageType('ModelValidationRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELVALIDATIONREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ModelValidationRequest)
  })
_sym_db.RegisterMessage(ModelValidationRequest)

ModelValidation = _reflection.GeneratedProtocolMessageType('ModelValidation', (_message.Message,), {
  'DESCRIPTOR' : _MODELVALIDATION,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ModelValidation)
  })
_sym_db.RegisterMessage(ModelValidation)

ModelRequest = _reflection.GeneratedProtocolMessageType('ModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _MODELREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ModelRequest)
  })
_sym_db.RegisterMessage(ModelRequest)

ModelResponse = _reflection.GeneratedProtocolMessageType('ModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _MODELRESPONSE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ModelResponse)
  })
_sym_db.RegisterMessage(ModelResponse)

GetGlobalModelRequest = _reflection.GeneratedProtocolMessageType('GetGlobalModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETGLOBALMODELREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.GetGlobalModelRequest)
  })
_sym_db.RegisterMessage(GetGlobalModelRequest)

GetGlobalModelResponse = _reflection.GeneratedProtocolMessageType('GetGlobalModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETGLOBALMODELRESPONSE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.GetGlobalModelResponse)
  })
_sym_db.RegisterMessage(GetGlobalModelResponse)

Heartbeat = _reflection.GeneratedProtocolMessageType('Heartbeat', (_message.Message,), {
  'DESCRIPTOR' : _HEARTBEAT,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.Heartbeat)
  })
_sym_db.RegisterMessage(Heartbeat)

ClientAvailableMessage = _reflection.GeneratedProtocolMessageType('ClientAvailableMessage', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTAVAILABLEMESSAGE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ClientAvailableMessage)
  })
_sym_db.RegisterMessage(ClientAvailableMessage)

ListClientsRequest = _reflection.GeneratedProtocolMessageType('ListClientsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTCLIENTSREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ListClientsRequest)
  })
_sym_db.RegisterMessage(ListClientsRequest)

ClientList = _reflection.GeneratedProtocolMessageType('ClientList', (_message.Message,), {
  'DESCRIPTOR' : _CLIENTLIST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ClientList)
  })
_sym_db.RegisterMessage(ClientList)

Client = _reflection.GeneratedProtocolMessageType('Client', (_message.Message,), {
  'DESCRIPTOR' : _CLIENT,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.Client)
  })
_sym_db.RegisterMessage(Client)

ReassignRequest = _reflection.GeneratedProtocolMessageType('ReassignRequest', (_message.Message,), {
  'DESCRIPTOR' : _REASSIGNREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ReassignRequest)
  })
_sym_db.RegisterMessage(ReassignRequest)

ReconnectRequest = _reflection.GeneratedProtocolMessageType('ReconnectRequest', (_message.Message,), {
  'DESCRIPTOR' : _RECONNECTREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ReconnectRequest)
  })
_sym_db.RegisterMessage(ReconnectRequest)

Parameter = _reflection.GeneratedProtocolMessageType('Parameter', (_message.Message,), {
  'DESCRIPTOR' : _PARAMETER,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.Parameter)
  })
_sym_db.RegisterMessage(Parameter)

ControlRequest = _reflection.GeneratedProtocolMessageType('ControlRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONTROLREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ControlRequest)
  })
_sym_db.RegisterMessage(ControlRequest)

ControlResponse = _reflection.GeneratedProtocolMessageType('ControlResponse', (_message.Message,), {
  'DESCRIPTOR' : _CONTROLRESPONSE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ControlResponse)
  })
_sym_db.RegisterMessage(ControlResponse)

ReportResponse = _reflection.GeneratedProtocolMessageType('ReportResponse', (_message.Message,), {
  'DESCRIPTOR' : _REPORTRESPONSE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ReportResponse)
  })
_sym_db.RegisterMessage(ReportResponse)

ConnectionRequest = _reflection.GeneratedProtocolMessageType('ConnectionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTIONREQUEST,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ConnectionRequest)
  })
_sym_db.RegisterMessage(ConnectionRequest)

ConnectionResponse = _reflection.GeneratedProtocolMessageType('ConnectionResponse', (_message.Message,), {
  'DESCRIPTOR' : _CONNECTIONRESPONSE,
  '__module__' : 'fedn.common.net.grpc.fedn_pb2'
  # @@protoc_insertion_point(class_scope:grpc.ConnectionResponse)
  })
_sym_db.RegisterMessage(ConnectionResponse)

_MODELSERVICE = DESCRIPTOR.services_by_name['ModelService']
_CONTROL = DESCRIPTOR.services_by_name['Control']
_REDUCER = DESCRIPTOR.services_by_name['Reducer']
_CONNECTOR = DESCRIPTOR.services_by_name['Connector']
_COMBINER = DESCRIPTOR.services_by_name['Combiner']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STATUSTYPE._serialized_start=2412
  _STATUSTYPE._serialized_end=2544
  _CHANNEL._serialized_start=2547
  _CHANNEL._serialized_end=2681
  _MODELSTATUS._serialized_start=2683
  _MODELSTATUS._serialized_end=2753
  _ROLE._serialized_start=2755
  _ROLE._serialized_end=2811
  _COMMAND._serialized_start=2813
  _COMMAND._serialized_end=2887
  _CONNECTIONSTATUS._serialized_start=2889
  _CONNECTIONSTATUS._serialized_end=2962
  _RESPONSE._serialized_start=41
  _RESPONSE._serialized_end=99
  _STATUS._serialized_start=102
  _STATUS._serialized_end=370
  _STATUS_LOGLEVEL._serialized_start=304
  _STATUS_LOGLEVEL._serialized_end=370
  _MODELUPDATEREQUEST._serialized_start=373
  _MODELUPDATEREQUEST._serialized_end=544
  _MODELUPDATE._serialized_start=547
  _MODELUPDATE._serialized_end=722
  _MODELVALIDATIONREQUEST._serialized_start=725
  _MODELVALIDATIONREQUEST._serialized_end=922
  _MODELVALIDATION._serialized_start=925
  _MODELVALIDATION._serialized_end=1093
  _MODELREQUEST._serialized_start=1096
  _MODELREQUEST._serialized_end=1233
  _MODELRESPONSE._serialized_start=1235
  _MODELRESPONSE._serialized_end=1328
  _GETGLOBALMODELREQUEST._serialized_start=1330
  _GETGLOBALMODELREQUEST._serialized_end=1415
  _GETGLOBALMODELRESPONSE._serialized_start=1417
  _GETGLOBALMODELRESPONSE._serialized_end=1521
  _HEARTBEAT._serialized_start=1523
  _HEARTBEAT._serialized_end=1564
  _CLIENTAVAILABLEMESSAGE._serialized_start=1566
  _CLIENTAVAILABLEMESSAGE._serialized_end=1653
  _LISTCLIENTSREQUEST._serialized_start=1655
  _LISTCLIENTSREQUEST._serialized_end=1737
  _CLIENTLIST._serialized_start=1739
  _CLIENTLIST._serialized_end=1781
  _CLIENT._serialized_start=1783
  _CLIENT._serialized_end=1831
  _REASSIGNREQUEST._serialized_start=1833
  _REASSIGNREQUEST._serialized_end=1942
  _RECONNECTREQUEST._serialized_start=1944
  _RECONNECTREQUEST._serialized_end=2043
  _PARAMETER._serialized_start=2045
  _PARAMETER._serialized_end=2084
  _CONTROLREQUEST._serialized_start=2086
  _CONTROLREQUEST._serialized_end=2170
  _CONTROLRESPONSE._serialized_start=2172
  _CONTROLRESPONSE._serialized_end=2242
  _REPORTRESPONSE._serialized_start=2244
  _REPORTRESPONSE._serialized_end=2326
  _CONNECTIONREQUEST._serialized_start=2328
  _CONNECTIONREQUEST._serialized_end=2347
  _CONNECTIONRESPONSE._serialized_start=2349
  _CONNECTIONRESPONSE._serialized_end=2409
  _MODELSERVICE._serialized_start=2964
  _MODELSERVICE._serialized_end=3086
  _CONTROL._serialized_start=3089
  _CONTROL._serialized_end=3386
  _REDUCER._serialized_start=3388
  _REDUCER._serialized_end=3474
  _CONNECTOR._serialized_start=3477
  _CONNECTOR._serialized_end=3904
  _COMBINER._serialized_start=3907
  _COMBINER._serialized_end=4509
# @@protoc_insertion_point(module_scope)
